/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.hadoop.hdfs.tools;

import java.io.IOException;
import java.io.PrintStream;
import java.net.InetSocketAddress;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hdfs.HdfsConfiguration;
import org.apache.hadoop.hdfs.protocolPB.GetUserMappingsProtocolClientSideTranslatorPB;
import org.apache.hadoop.hdfs.server.namenode.NameNode;
import org.apache.hadoop.security.UserGroupInformation;
import org.apache.hadoop.tools.GetGroupsBase;
import org.apache.hadoop.tools.GetUserMappingsProtocol;
import org.apache.hadoop.util.ToolRunner;

/**
 * HDFS implementation of a tool for getting the groups which a given user
 * belongs to.
 */
public class GetGroups extends GetGroupsBase {

  static{
    HdfsConfiguration.init();
  }

  
  GetGroups(Configuration conf) {
    super(conf);
  }

  GetGroups(Configuration conf, PrintStream out) {
    super(conf, out);
  }
  
  @Override
  protected InetSocketAddress getProtocolAddress(Configuration conf)
      throws IOException {
    return NameNode.getAddress(conf);
  }
  
  @Override
  protected GetUserMappingsProtocol getUgmProtocol() throws IOException {
    return new GetUserMappingsProtocolClientSideTranslatorPB(
        NameNode.getAddress(getConf()), UserGroupInformation.getCurrentUser(),
        getConf());
  }

  public static void main(String[] argv) throws Exception {
    int res = ToolRunner.run(new GetGroups(new HdfsConfiguration()), argv);
    System.exit(res);
  }
}